#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var node_fetch_1 = require("node-fetch");
var introspectionQuery_1 = require("graphql/utilities/introspectionQuery");
var buildClientSchema_1 = require("graphql/utilities/buildClientSchema");
var minimist = require("minimist");
var chalk = require("chalk");
var fs = require("fs");
var graphql_1 = require("graphql");
var utilities_1 = require("graphql/utilities");
var transformChangeType_1 = require("./utils/transformChangeType");
var transformChangeDescription_1 = require("./utils/transformChangeDescription");
var Table = require("cli-table2");
var _a = require('../package.json'), version = _a.version, name = _a.name;
var usage = "\n  " + chalk.bold('Check if the remote server fulfills the supplied GraphQL contract file') + "\n\n  Usage: graphql-contract-test ENDPOINT_URL client_schema_file\n\n  Options:\n    --header, -h    Add a custom header (ex. 'Authorization=Bearer ...'), can be used multiple times\n";
var intro = "  GraphQL Contract Test v" + version + "\n";
function main() {
    return __awaiter(this, void 0, void 0, function () {
        var argv, endpoint, contractFile, headers, implementation, contract, breakingChanges, table;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    console.log(intro);
                    argv = minimist(process.argv.slice(2));
                    if (argv._.length < 2) {
                        console.log(usage);
                        process.exit(1);
                    }
                    endpoint = argv._[0];
                    contractFile = argv._[1];
                    headers = parseHeaderOptions(argv);
                    return [4 /*yield*/, getImplementedSchema(endpoint, headers)];
                case 1:
                    implementation = _a.sent();
                    contract = getContractSchema(contractFile);
                    breakingChanges = utilities_1.findBreakingChanges(contract, implementation);
                    if (breakingChanges.length === 0) {
                        console.log(chalk.bold.green('  ✨  The server appears to implement the schema you provided'));
                        process.exit(0);
                    }
                    console.log("  \uD83D\uDCA9  " + chalk.bold.red('Breaking changes were detected\n'));
                    table = buildResultsTable(breakingChanges);
                    console.log(table.toString());
                    process.exit(1);
                    return [2 /*return*/];
            }
        });
    });
}
function buildResultsTable(breakingChanges) {
    var table = new Table({
        head: ['Issue', 'Description'],
    });
    breakingChanges.forEach(function (change) {
        table.push([transformChangeType_1.transformChangeType(change.type), transformChangeDescription_1.transformChangeDescription(change.description)]);
    });
    return table;
}
function getContractSchema(expectedSchemaFile) {
    var data = fs.readFileSync(expectedSchemaFile, 'utf8');
    return graphql_1.buildSchema(data);
}
function getImplementedSchema(endpoint, headers) {
    return __awaiter(this, void 0, void 0, function () {
        var response, _a, data, errors;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, node_fetch_1.default(endpoint, {
                        method: 'POST',
                        headers: headers,
                        body: JSON.stringify({ query: introspectionQuery_1.introspectionQuery }),
                    })];
                case 1:
                    response = _b.sent();
                    return [4 /*yield*/, response.json()];
                case 2:
                    _a = _b.sent(), data = _a.data, errors = _a.errors;
                    if (errors) {
                        throw new Error(JSON.stringify(errors, null, 2));
                    }
                    return [2 /*return*/, buildClientSchema_1.buildClientSchema(data)];
            }
        });
    });
}
function parseHeaderOptions(argv) {
    var defaultHeaders = {
        'Content-Type': 'application/json',
        'User-Agent': name + " v" + version
    };
    return toArray(argv['header'])
        .concat(toArray(argv['h']))
        .reduce(function (obj, header) {
        var _a = header.split('='), key = _a[0], value = _a[1];
        obj[key] = value;
        return obj;
    }, defaultHeaders);
}
function toArray(value) {
    if (value === void 0) { value = []; }
    return Array.isArray(value) ? value : [value];
}
main().catch(function (e) {
    console.log("" + chalk.bold.red(e.message));
    process.exit(1);
});
//# sourceMappingURL=index.js.map